<?php

namespace Teknisa\PLA\Modules\Cardapio\Controllers;

use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Teknisa\GER\Utilities\Environment;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DTO;
use \Doctrine\ORM\EntityManager;
use Exception;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;

class Pla00150AtualizarDataDeReferenciaParaCustos extends \Teknisa\GER\Controllers\TeknisaCrud {

    private   $entityManager;
    protected $dataSourceManager;
    protected $environment;
    protected $parameterBag;
    protected $pla00150AtuDtRefCusService;
    protected $crudService;

    protected $dataSourceName = 'cardames';
    protected $cdgrupoper;
    protected $filiais;
    protected $servicos;
    protected $mes;
    protected $dtreferencia;

    public function __construct($crudService,  $parameterBag, $entityManager, Manager $dataSourceManager, Environment $environment,  $pla00150AtuDtRefCusService) {
        $this->crudService                 = $crudService;
        $this->entityManager               = $entityManager;
        $this->dataSourceManager           = $dataSourceManager;
        $this->environment                 = $environment;
        $this->parameterBag                = $parameterBag;
		$this->pla00150AtuDtRefCusService  = $pla00150AtuDtRefCusService;

        parent::__construct($crudService, $parameterBag);
    }

    public function validacoes(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $this->verificaFiliaisSelecionadas($request, $response);
            $this->verificaDataProc($request, $response);
            $this->verificaPermissaoOperador($request, $response);

        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 1));
        }
    }

    /* Validação Filiais Selecionadas */
    public function verificaFiliaisSelecionadas($request, $response) {
        $row = $request->getRow();
        $grupfili = empty($row['NMGRUPFILI']) ? 'T' : $row['CDGRUPFILI'];
        $filiais = $this->findAllFilial($grupfili);
        if ($filiais == "_") {
            throw new \Exception("Grupo de Unidades selecionado não possui unidades.", 1);
        }
    }

    /* Validação Data de Processamento */
    public function verificaDataProc($request, $response) {
        $row = $request->getRow();
        $cdfilial = join("_", $this->getFiliais($row));
        $data = $row['_MES'];
        $nrorg = $this->environment->getNrOrg();
        $this->parameterBagColumns = array('CDFILIAL', 'DATA', 'NRORG');

        $filterCriteria = $this->pla00150AtuDtRefCusService->verificaDataProc($cdfilial, $data, $nrorg);
        $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);
        $dataSet = $this->dataSourceManager->findBy($newFilterCriteria);
        return $this->validaDataProc($response, $dataSet);
    }

    public function validaDataProc($response, $data) {
        $filiais = $data->getRows();
        $filiaisForaDataProc =  array();
        if (!empty($filiais)) {
            foreach ($filiais as $filial) {
               array_push($filiaisForaDataProc, $filial['CDFILIAL']." - ".$filial['NMFILIAL']);
            }
            $response->addDataSet(new DataSet ('filiaisForaDataProc', $filiaisForaDataProc));
            throw new Exception("Operação bloqueada");
        }
    }

    /* Validação Permissão Operador */
    public function verificaPermissaoOperador($request, $response) {
        $this->verificaGrupPermissao();
        if ($this->cdgrupoper) {
            $this->verificaGrupOper($request, $response);
            $this->validaPermissaoOperador($response);
        }
    }

    public function verificaGrupPermissao() {
        $nrorg = $this->environment->getNrOrg();
        $cdoperador = $this->environment->getCdOperador();
        $idautgrupop = "RCUS";
        $this->cdgrupoper = $this->pla00150AtuDtRefCusService->getAutgrupop($idautgrupop, $nrorg);
    }

    public function verificaGrupOper(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        $nrorg = $this->environment->getNrOrg();
        $cdoperador = $row['CDOPERADOR'];
        $this->dataSourceName = 'opergrupop';
        $filterCriteria = new FilterCriteria($this->dataSourceName , array());
        $filterCriteria->addCondition("NRORG", "=", $nrorg);
        $filterCriteria->addCondition("CDOPERADOR", "=", $cdoperador);
        $filterCriteria->addCondition("CDGRUPOPER", "=", $this->cdgrupoper);
        $this->buildFilterAndFind($request, $response, $filterCriteria, "opergrupop");
    }


    public function validaPermissaoOperador($response) {
        if (empty($response->getDataSets()[0]->getRows())) {
            $response->setDataSets(array());
            $response->addDataSet(new DataSet ('sempermissao', array()));
            throw new Exception("A atualização da data de referência para custos depende de autorização");
        } else {
            $response->setDataSets(array());
        }
    }

    public function validaAutorizacaoOper(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $this->verificaGrupPermissao();
            $this->verificaGrupOper($request, $response);
            if (empty($response->getDataSets()[0]->getRows())) {
                $response->setDataSets(array());
                throw new Exception("Operador não pertence ao grupo responsável pela autorização.");
            } else {
                $row = $request->getRow();
                $operadorInformado = $this->entityManager->getRepository('\Teknisa\Model\Geral\Entities\Operador')->findOneBy(array("cdoperador" => $row['CDOPERADOR'], "nrorg" => $this->environment->getNrOrg()));
                $senhaInformada = md5($row['SENHA']);
                $senhaCorreta = $operadorInformado->getCdsenhaoperweb();
                if ($senhaInformada == $senhaCorreta) {
                    $response->setDataSets(array());
                } else {
                    $response->setDataSets(array());
                    throw new Exception("Senha Inválida");
                }
            }
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 1));
        }
    }

    /* Atualiza Data Referência Custo */
    public function atualizaDtRefCust(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            if($row['CHECK_NMSERVICO'] == 'T'){
                $this->findAllServicos($row);
            } else {
                $this->servicos = $row['NMSERVICO'];
            }
            $this->atualizaCardames($request, $response);
            $this->verificaGravaPreCusto($row);
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 1));
        }
    }

    public function atualizaCardames($request, $response) {
        try {
            $row = $request->getRow();
            $rowToSave = array(
                "CDFILIAL"      => '',
                "CDSERVFILI"    => '',
                "DTMESCARDAPIO" => $row['_MES'],
                "DTREFECUSTMES" => $row['DTREF'],
                "NRTIPOCUSTO"   => $row['NRTIPOCUSTO'],
                "NRORG"         => $this->environment->getNrOrg(),
                "__is_new"      => ''
            );
            $rowsToSave = array();
            foreach ($this->servicos as $servico) {
                $rowToSave['CDFILIAL'] = $servico['CDFILIAL'];
                $rowToSave['CDSERVFILI'] = $servico['CDSERVFILI'];
                $rowToSave['__is_new'] = $this->isNewCardames($servico['CDFILIAL'], $servico['CDSERVFILI'], $row['_MES']);
                array_push($rowsToSave, $rowToSave);
            }
            $dataSave = new DataSource\DataSet('cardames', $rowsToSave);
            $requestSave = new DTO\Request\DataSet($dataSave, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
            parent::save($requestSave, $response);
            
            //foreach GERA_HISTCARDAMES
            $this->entityManager->beginTransaction();
            foreach ($this->servicos as $servico) {
                $rowToSave['CDFILIAL'] = $servico['CDFILIAL'];
                $rowToSave['CDSERVFILI'] = $servico['CDSERVFILI'];
                $this->geraHistCardaMes($rowToSave);
            }
            $this->entityManager->commit();
            
        } catch(\Exception $e) {
            $this->entityManager->rollback();
            $response->setError(new DTO\Response\Error($e->getMessage(), 500));
        }
    }
    
    private function geraHistCardaMes($row) {
    	$procedure = new StoredProcedure($this->entityManager->getConnection(), 'GERA_HISTCARDAMES');
        
        $procedure->addParam(new Param('P_CDFILIAL',    Param::PARAM_INPUT,  $row['CDFILIAL'],                    Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDSERVFILI',  Param::PARAM_INPUT,  $row['CDSERVFILI'],                  Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTMES',       Param::PARAM_INPUT,  $row['DTMESCARDAPIO'],               Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDTIPOCUSTO', Param::PARAM_INPUT,  $row['NRTIPOCUSTO'],                 Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTREFECUSTO', Param::PARAM_INPUT,  $row['DTREFECUSTMES'],               Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRORG',       Param::PARAM_INPUT,  $this->environment->getNrOrgTrab(),  Param::PARAM_TYPE_INT));
        $procedure->addParam(new Param('P_CDOPERADOR',  Param::PARAM_INPUT,  $this->environment->getCdoperador(), Param::PARAM_TYPE_STR));
        
        $result = $procedure->execute();
        
        return $result;
    }

    public function isNewCardames($cdfilial, $cdservfili, $mes){
        $conditions = array(
            array(
                'columnName' => 'CDFILIAL',
                'operator' => '=',
                'value' => $cdfilial
            ),
            array(
                'columnName' => 'CDSERVFILI',
                'operator' => '=',
                'value' => $cdservfili
            ),
            array(
                'columnName' => 'DTMESCARDAPIO',
                'operator' => '=',
                'value' => $mes
            )
        );
        $filterCriteria = new FilterCriteria('cardames', $conditions);
        $dataSet = $this->dataSourceManager->findBy($filterCriteria);
        return empty($dataSet->getRows());

    }

    public function findAllServicos($row){
        $grupfili = empty($row['NMGRUPFILI']) ? 'T' : $row['CDGRUPFILI'];
        $filial = ($row['CHECK_NMFILIAL'] == 'T') ? $this->findAllFilial($grupfili) : '_'.implode('_', $row['NMFILIAL']).'_';
        $conditions = array(
            array(
                'columnName' => 'CDFILIAL',
                'operator' => '=',
                'value' => $filial
            ),
            array(
                'columnName' => 'IDTIPOSERV',
                'operator' => '=',
                'value' => 'N'
            ),
            array(
                'columnName' => 'CDOPERADOR',
                'operator' => '=',
                'value' => $this->environment->getCdOperador()
            ),
            array(
                'columnName' => 'CDGRUPFILI',
                'operator' => '=',
                'value' => $grupfili
            ),
            array(
                'columnName' => 'NRORG',
                'operator' => '=',
                'value' => $this->environment->getNrOrg()
            )
        );
        $filterCriteria = new FilterCriteria('servicos_por_agrupfili_multi_filiais', $conditions);
        $paramBag = array("NRORG", "CDFILIAL", "CDOPERADOR", "CDGRUPFILI");
        $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $paramBag);
        $dataSet = $this->dataSourceManager->findBy($newFilterCriteria);
        $this->servicos = $this->buildArray($dataSet->getRows(), array('CDFILIAL', 'CDSERVFILI'));
        $row['NMSERVICO'] = $this->servicos;
    }

    public function findAllFilial($grupfili){
        $conditions = array(
            array(
                'columnName' => 'CDGRUPFILI',
                'operator' => '=',
                'value' => $grupfili
            ),
            array(
                'columnName' => 'CDOPERADOR',
                'operator' => '=',
                'value' => $this->environment->getCdOperador()
            ),
            array(
                'columnName' => 'NRORG',
                'operator' => '=',
                'value' => $this->environment->getNrOrg()
            )
        );
        $filterCriteria = new FilterCriteria('filial_operador_grupfili', $conditions);
        $paramBag = array("NRORG", "CDOPERADOR", "CDGRUPFILI");
        $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $paramBag);
        $dataSet = $this->dataSourceManager->findBy($newFilterCriteria);
        $rows = $dataSet->getRows();

        $concatFiliais = '_';
        foreach($rows as $filial){
            $concatFiliais = $concatFiliais.$filial['CDFILIAL'].'_';
        }
        return $concatFiliais;
    }
    public function verificaGravaPreCusto($row) {
        $gravaprecusto = $this->pla00150AtuDtRefCusService->getProcedure();
        if (!empty($gravaprecusto)) {
            $dtinicial = $row['DTINICIAL'];
            $dtfinal = $row['DTFINAL'];
            $codtipocusto = '6';
            $codprodinicial = '1';
            $codprodfinal = '3zzzzzzzzz';
            $dtrefpesquisa = $row['DTATUAL'];
            $dadosProc = $this->filterConcatFiliaisServicos($row['array_filiais'], $row['array_servicos']);
            foreach ($dadosProc as $dados) {
                $cdfilial = $dados['CDFILIAL'];
                $servicos = $dados['CDSERVFILI'];
                $this->pla00150AtuDtRefCusService->GRAVAPRECUSTO($dtinicial, $dtfinal, $codtipocusto, $codprodinicial, $codprodfinal, $dtrefpesquisa, $cdfilial, $servicos);
            }
        }
    }

    /* Gerais */
    public function buildFilterAndFind($request, $response, $filterCriteria, $dataSourceName) {
        $request = new DTO\Request\Filter(
            $filterCriteria,
            $request->getMethod(),
            $dataSourceName,
            $request->getUserId()
        );
        parent::find($request, $response);
    }

    public function getFiliais($row) {
        $cdfilial = $row['array_filiais'];
        if (!is_array($cdfilial) && $cdfilial == 'T') {
            $cdgrupfili = isset($row['CDGRUPFILI']) ? $row['CDGRUPFILI'] : 'T';
            $nrorg = $this->environment->getNrOrg();
            $cdoperador = $this->environment->getCdOperador();
            $this->parameterBagColumns = array('CDGRUPFILI', 'CDOPERADOR', 'NRORG');

            $filterCriteria = $this->pla00150AtuDtRefCusService->getFiliais($cdgrupfili, $cdoperador, $nrorg);
            $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);
            $dataset = $this->dataSourceManager->findBy($newFilterCriteria);
            $cdfilial = $this->buildArray($dataset->getRows(), 'CDFILIAL');
        }
        return $cdfilial;
    }

    public function getServicos($row, $filiais) {
        $servicos = $row['array_servicos'];
        if (!is_array($servicos) && $servicos == 'T') {
            $cdfilial = $filiais;
            $idtiposerv = 'N';
            $nrorg = $this->environment->getNrOrg();
            $filterCriteria = $this->pla00150AtuDtRefCusService->getServicos($cdfilial, $idtiposerv, $nrorg);
            $dataset = $this->dataSourceManager->findBy($filterCriteria);
            $servicos = $this->buildArray($dataset->getRows(), array('CDFILIAL', 'CDSERVFILI'));
        }
        return $servicos;
    }

    public function buildArray($array, $pos) {
        $servicos = array();
        if (is_array($array)) {
            $x = sizeof($array);
            foreach ($array as $item) {
                if (is_array($pos)) {
                    foreach ($pos as $p) {
                        $servicos[$x][$p] =$item[$p];
                    }
                    $x--;
                } else {
                    $servicos[] = $item[$pos];
                }
            }
            return $servicos;
        }
    }

    public function filterConcatFiliaisServicos($filiais, $servicos) {
        $array = array();
        foreach ($filiais as $filial) {
            $concat = '';
            foreach ($servicos as $servico) {
                if ($servico['CDFILIAL'] == $filial) {
                    $concat = $concat.'_'.$servico['CDSERVFILI'];
                }
            }
            if (isset($concat)) {
                $prev['CDFILIAL'] = $filial;
                $prev['CDSERVFILI'] = $concat;
                array_push($array, $prev);
            }
        }
        return $array;
    }
}